/***********************************************************
 * $Id: HttpResource.java 945 2014-03-21 08:17:26Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 17.02.2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.io.Closeable;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.MessageDigest;
import java.util.Date;

/**
 * A static resource to be delivered to the client by our
 * {@link ResourceServlet}.
 */
public interface HttpResource extends Closeable {

    /**
     * Copy the resource to an output stream.
     * 
     * @param os The output stream to write to.
     * @throws IOException Upon I/O errors.
     */
    public void writeToStream(OutputStream os) throws IOException;
    
    /**
     * Update this resource from a stream. Optional operation
     * only implemented, when {@link #isUpdateable()} returns
     * <code>true</code>.
     * 
     * @param is The input stream to read the updated content from.
     * @throws IOException
     */
    public void readFromStream(InputStream is) throws IOException;
    
    /**
     * @param md A message digest to update with the contents of the
     *           resource.
     * @return The number of bytes written to the digest, aka the
     *           content-length of the resource.
     * @throws IOException Upon I/O errors.
     */
    public long makeDigest(MessageDigest md) throws IOException;
    
    /**
     * @return The last modified date of the resource or
     *         <code>null</code>, if the last modification date is unknown.
     */
    public Date getLastModified();
    
    /**
     * @return Whether {@link #writeToStream(OutputStream)} and/or
     *           {@link #makeDigest(MessageDigest)} may be called
     *           multiple times or the resource has to be resolved
     *           each time it is going to be used.
     */
    public boolean isReusable();
    
    /**
     * @return Whether this resource may be updated and
     *         {@link #readFromStream(InputStream)} is implemented.
     */
    public boolean isUpdateable();
}
