/***********************************************************
 * $Id: LocaleHelper.java 869 2013-07-23 12:22:01Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 16.06.2011
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http;

import java.util.Locale;


/**
 * This class provides static functionality for handling locales.
 */
public abstract class LocaleHelper {

    private static final AcceptMatcher LOCALE_MATCHER = new AcceptMatcher() {

		public double matchItem(String accepted, String provided) {
			
			int apos=0,ppos=0;
			int napos,nppos;
			
			int nitems = 0;
			
			while (true) {
				
				napos = accepted.indexOf('-',apos);
				if (napos < 0)
					napos = accepted.indexOf('_',apos);
				
				
				
				nppos = provided.indexOf('-',ppos);
				if (nppos < 0)
					nppos = provided.indexOf('_',ppos);
				
				String apart = napos < 0 ? accepted.substring(apos+1) : accepted.substring(apos+1,napos);
				String ppart = nppos < 0 ? provided.substring(ppos+1) : provided.substring(ppos+1,nppos);
				
				if (apart.compareToIgnoreCase(ppart) != 0) {
					
					if (nitems > 0)
						// correct language, wrong dialect.
						return 0.25;
					else
						return 0.0;
				}
				
				++nitems;
				
				// accepted at end
				if (napos < 0) {
					if (nppos < 0) {
						// provided at end: perfect match
						return 1.0;
					}
					else {
						// accepted has less items than provided, so
						// server provides a dialects for a client seeking
						// a general language.
						return 0.6;
					}
				}
				
				if (nppos < 0) {
					// provided has less items than accepted, so
					// server does not care about dialects or it may provide
					// a better dialected version.
					return 0.5;
				}
				
				apos = napos+1;
				ppos = nppos+1;
			}
		}
    };
    
    /**
     * @return A matcher for locales in an <code>Accept-Language</code> header.
     */
    public static final AcceptMatcher getLocaleMatcher() {
		
		return LOCALE_MATCHER;
	}
	
    /**
     * @param acceptLanguage The content of an Accept-Language header (see section 14.3 of RFC2616).
     * @param supportedLocales A list of supported encodings to choose from.
     * @return The locale selected to use for the given browser of <code>null</code>, if no
     *             matching locale could be found.
     */
    public static String selectLocale(String acceptLanguage, String[] supportedLocales) {
        
       return AcceptHelper.selectAcceptItem(acceptLanguage,supportedLocales,LOCALE_MATCHER);
    }

	/**
	 * Convert an XML schema language to a java locale.
	 * 
	 * Examples:
	 * <pre>
	 * fromXsLanguage("de-AT") == new Locale("de","AT")
	 * fromXsLanguage("de") == new Locale("de")
	 * </pre>
	 * 
	 * @param lang A language as specified by the XML schema language type.
	 * @return A JAVA locale.
	 */
	public static Locale localeFromXsLanguage(String lang) {
		
		String parts[] = lang.split("-",2);
		
		if (parts.length == 1)
			return new Locale(parts[0]);
		else
			return new Locale(parts[0],parts[1]);
	}
	
	/**
	 * Convert a java locale to an XML schema language.
	 * 
	 * Examples:
	 * <pre>
	 * toXsLanguage(new Locale("de","AT")) == "de-AT"
	 * toXsLanguage(new Locale("de")) == "de"
	 * </pre>
	 * 
	 * @param locale A JAVA locale.
	 * @return An XML language.
	 */
	public static String toXsLanguage(Locale locale) {
		
		if (locale.getCountry() == null || locale.getCountry().isEmpty())
			return locale.getLanguage();
		else
			return locale.getLanguage() + "-" + locale.getCountry();
		
	}

}
