/***********************************************************
 * $Id: DefaultHttpContext.java 796 2013-02-19 16:28:09Z util $
 * 
 * HTTP Utility classes of the clazzes.org project
 * http://www.clazzes.org
 *
 * Created: 24.09.2012
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 ***********************************************************/

package org.clazzes.util.http.osgi;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.Reader;
import java.net.URL;
import java.util.Collections;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.osgi.service.http.HttpContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * <p>A default HTTP context, which my be shared among bundles using
 * the static instance returned by {@link DefaultHttpContext#getInstance()}.
 * </p>
 * 
 * <p>The method {@link #getResource(String)} look up the resource under the
 * context class loader of the current thread under normative path
 * <code>/OSGI-INF/webapp/</code>.
 * </p>
 */
public class DefaultHttpContext implements HttpContext {

	/**
	 * The system property to check for an alternative mime.types file to
	 * parse instead of the built-in classpath resource.
	 */
	public static final String MIME_TYPES_PROPERTY = "org.clazzes.util.http.mimeTypes";
	private static final String MIME_TYPES_RESOURCE = "mime.types";

	private static final Logger log = LoggerFactory.getLogger(DefaultHttpContext.class);

	private static final Map<String,String> extensionMimeTypes;
	
	static {
		
		InputStream is = null;

		try {
			
			String fn = System.getProperty(MIME_TYPES_PROPERTY);
			
			if (fn == null) {
			
				if (log.isDebugEnabled()) {
					log.debug("Parsing MIME type mapping from built-in class path resource [{}] in [{}].",
							MIME_TYPES_RESOURCE,DefaultHttpContext.class.getPackage());
				}

				is = DefaultHttpContext.class.getResourceAsStream(MIME_TYPES_RESOURCE);
			}
			else {
				if (log.isDebugEnabled()) {
					log.debug("Parsing MIME type mapping from file [{}] configured by property [{}].",fn,MIME_TYPES_PROPERTY);
				}
				
				is = new FileInputStream(fn);
			}
			Reader r = new InputStreamReader(is,"UTF-8");
			LineNumberReader lnr = new LineNumberReader(r);
			
			extensionMimeTypes = new HashMap<String, String>(128);
			
			String line;
			
			while ((line = lnr.readLine()) != null) {
				
			  if (line.length() == 0) {
			    continue;
			  }
			  if (line.charAt(0) == '#' || line.charAt(0) == ';') {
			    continue;
			  }
			  
				String[] parts = line.split("[ \t]+");
				
				if (parts.length > 1) {
					
					for (int i=1;i<parts.length;++i) {
						String ov = extensionMimeTypes.put(parts[i],parts[0]);
						
						if (log.isDebugEnabled()) {
							log.debug("Adding extension mapping [{}->{}]",parts[i],parts[0]);
						}
						
						if (ov != null) {
							log.warn("Duplicate extension [{}]",parts[i]);
						}
					}
				}
			}
			
			r.close();
			
		} catch (IOException e) {
			throw new RuntimeException("Error parsing resource [mime.types]");
		}
		finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException e) {
					log.warn("Error closing mime.types input stream",e);
				}
			}
		}
	}
	
	private static final HttpContext INSTANCE = new DefaultHttpContext();
	
	/**
	 * @return The static, shared instance for web application, which want to share HTTP sessions.
	 */
	public static final HttpContext getInstance() {
		return INSTANCE;
	}
		
	/**
	 * @return A static, unmodifiable view on the map of MIME types
	 *         used by {@link #getMimeType(String)} mapping extensions
	 *         to MIME types.
	 */
	public static final Map<String,String> getExtensionMimeTypes() {
		return Collections.unmodifiableMap(extensionMimeTypes);
	}
	
	/* (non-Javadoc)
	 * @see org.osgi.service.http.HttpContext#handleSecurity(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	public boolean handleSecurity(HttpServletRequest request,
			HttpServletResponse response) throws IOException {
		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.osgi.service.http.HttpContext#getResource(java.lang.String)
	 */
	@Override
	public URL getResource(String name) {

		String resourcePath;
		
		if (name.startsWith("/")) {
    		resourcePath = "/OSGI-INF/webapp"+name;
    	} else {
    		resourcePath = "/OSGI-INF/webapp/"+name;
    	}

		if (log.isDebugEnabled()) {
    		log.debug("fetching resource from ["+resourcePath+"]");
		}
		
		return Thread.currentThread().getContextClassLoader().getResource(resourcePath);
	}

	/* (non-Javadoc)
	 * @see org.osgi.service.http.HttpContext#getMimeType(java.lang.String)
	 */
	@Override
	public String getMimeType(String name) {
		
        String mimeType = null;
        
        int dpos = name.lastIndexOf('.');
        
        if (dpos >= 0) {
        	
        	String ext = name.substring(dpos+1).toLowerCase(Locale.US);
        	mimeType = extensionMimeTypes.get(ext);
        	
        	if (mimeType == null && dpos > 0) {
        		
        		dpos = name.lastIndexOf('.',dpos-1);
        		
        		if (dpos >= 0) {
        			
        			ext = name.substring(dpos+1).toLowerCase(Locale.US);
        			mimeType = extensionMimeTypes.get(ext);
        		}
        	}
        }
        
        return mimeType;
	}

}
